/****************************************************************
*
* Copyright (C) 2004 SignalQuest, Inc.
*
*----------------------------------------------------------------
* All software and related documentation is provided "AS IS" and 
* without warranty or support of any kind and SignalQuest 
* expressly disclaims all other warranties, express or implied, 
* including, but not limited to, the implied warranties of 
* merchantability and fitness for a particular purpose. Under no 
* circumstances shall SignalQuest be liable for any 
* incidental, special or consequential damages that result from 
* the use or inability to use the software or related 
* documentation, even if SignalQuest has been advised of 
* the liability.
*
* Unless otherwise stated, software written and copyrighted by 
* SignalQuest is distributed as "freeware". You may use 
* and modify this software without any charge or restriction. 
* You may distribute to others, as long as the original author 
* is acknowledged.
*
*****************************************************************
* 
* Project:  SignalQuest Serial Communications DLL
*
* File:    SQincl.cpp
*
* History:
*   Version 1.71 (2004-11-10) pmt. Extensive fixes to thread usage.
*   Version 1.5  (2004-08-30)
*
****************************************************************/


// SQIncl.cpp : Defines the initialization routines for the DLL.
//




#include "stdafx.h"
#include "SQIncl.h"

#include "MyWin32Port.h"					// Nelson's derived RS232 port
#include "SQRS232Errors.h"					// SQ error defines
#include "RS232Messages.h"					// SQ windows message defines




#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif




//
//	Note!
//
//		If this DLL is dynamically linked against the MFC
//		DLLs, any functions exported from this DLL which
//		call into MFC must have the AFX_MANAGE_STATE macro
//		added at the very beginning of the function.
//
//		For example:
//
//		extern "C" BOOL PASCAL EXPORT ExportedFunction()
//		{
//			AFX_MANAGE_STATE(AfxGetStaticModuleState());
//			// normal function body here
//		}
//
//		It is very important that this macro appear in each
//		function, prior to any calls into MFC.  This means that
//		it must appear as the first statement within the 
//		function, even before any object variable declarations
//		as their constructors may generate calls into the MFC
//		DLL.
//
//		Please see MFC Technical Notes 33 and 58 for additional
//		details.
//


/////////////////////////////////////////////////////////////////////////////
// CSQInclApp

BEGIN_MESSAGE_MAP(CSQInclApp, CWinApp)
	//{{AFX_MSG_MAP(CSQInclApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CSQInclApp construction

// Constructor
CSQInclApp::CSQInclApp()
{
	// Check for memory leaks
	#ifdef _DEBUG
	m_msOld.Checkpoint();
	#endif
}


// Destructor
CSQInclApp::~CSQInclApp()
{
	// Check for memory leaks
	#ifdef _DEBUG
	m_msOld.DumpAllObjectsSince();
	m_msNew.Checkpoint();
	m_msDif.Difference(m_msOld, m_msNew);
	m_msDif.DumpStatistics();
	#endif
}


/////////////////////////////////////////////////////////////////////////////
// CSQInclApp User-Defined Functions

LONG CSQInclApp::Config(
	LONG liMode,
	LONG liBuffSize,
	LONG liRS232Timeout,
	LONG * liRetVal1,
	LONG * liRetVal2)
{
	LONG liErrorCode = 0;


	if(liMode == DATA_MODE_LOAD_DLL)
	{
		m_iPreLoaded = 1;
	}
	else
	{
		if((liMode >= DATA_MODE_SYNC) && (liMode <= DATA_MODE_ASYNC_EVENT))
		{	// For any valid data modes
			SetDataSyncMode(liMode);
		}

		if(liRS232Timeout > 0)
		{
			m_iTimeout = liRS232Timeout;
		}

		if(m_pPort != 0)
		{
			if((liMode == DATA_MODE_ASYNC_NO_EVENT) ||
				(liMode == DATA_MODE_ASYNC_EVENT))
			{
				m_iAsyncFirstTime = 1;
			}

			if(liMode == DATA_MODE_ASYNC_EVENT)
			{
				if(m_pPort != 0)
				{
					*liRetVal1 = (LONG)m_pPort->GetPacketReadyEvent();
				}
				else
				{
					liErrorCode = ERROR_CONFIG_PORT_NOT_OPEN;
				}
			}

			if(liMode == DATA_MODE_STOP)
			{
				m_pPort->FlushTXBuffer();
				m_pPort->FlushRXBuffer();

				m_pPort->SQFlushRX();
			}


			if(liMode == DIG_IO_TURN_ON_EVENTS)
			{
				*liRetVal1 = (LONG)m_pPort->GetCTSStatusEvent();
				*liRetVal2 = (LONG)m_pPort->GetDSRStatusEvent();

				m_pPort->TurnOnDigIO();
			}


			if(liMode == DIG_IO_TURN_OFF_EVENTS)
			{
				m_pPort->TurnOffDigIO();
			}
		}
		else
		{
			liErrorCode = ERROR_CONFIG_PORT_NOT_OPEN;
		}
	}


	return(liErrorCode);
}


LONG CSQInclApp::GetPreloadedState(void)
{
	return(m_iPreLoaded);
}


void CSQInclApp::CreateDataObjects(void)
{
	LONG liErrorCode = 0;


	// Initialize variables
	m_iDataSyncMode = DATA_MODE_STOP;
	m_iAsyncFirstTime = 1;
	m_iIsClosing = 0;
	m_iIsOpening = 0;
	m_iDeletePortRequest = 0;

	m_iPreLoaded = 0;
	m_iPollingMode = 0;
	m_iTimeout = DEFAULT_RS232_TIMEOUT;

	m_pPort = 0;

	m_CurrentPacket.usPacketCount = 0;
	m_CurrentPacket.ssCurrentErrorState = 0;
	m_CurrentPacket.usLatchedErrorState = 0;
	m_CurrentPacket.usValue1 = 0;
	m_CurrentPacket.usValue2 = 0;

	m_hPacketReadyEvent = 0;

	m_hDeletePortDoneEvent = ::CreateEvent(NULL, TRUE, TRUE, NULL);

	sprintf(m_szCOM, "\0");


	// If not successfull, cleanup
	if(liErrorCode != ERROR_RS232_SUCCESS)
	{
		// Fatal error, exit app
		PostQuitMessage(0);
	}
} // End CreateDataObjects()


void CSQInclApp::DestroyDataObjects(void)
{
	// If port is open
	if(m_pPort != 0)
	{
		// Delete port object
		DeletePort();
	}
}


void CSQInclApp::PeekPause(long liMilliseconds)
{
	long liTime1, liTime2, liDiff;


	liTime1 = ReadTime();
	liDiff = 0;


	while(liDiff < liMilliseconds)
	{
		liTime2 = ReadTime();

		liDiff = liTime2 - liTime1;

		if(liDiff < 0)
		{
			liTime1 = ReadTime();
		}

		MSG msg;

		if(::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
	}
}

//**********************************************************
LONG CSQInclApp::OpenPort(
	LPSTR lpszCOMPort,
	LONG  liBaudRate,
	LPSTR lpszParity,
	LONG  liWordSize,
	LONG  liStopBits,
	LONG  liXonXoff,
	LONG  liRtsCts,
	LONG  liDtrDsr)
{
	LONG liErrorCode;			// Local error code
	int len;					// Length of string
	char m_szCOM_long[10];		// Store currently open com port
	int iDestroyObject = 0;


	// Assume no error
	liErrorCode = 0;
	SetLastError(0);
	if (long Error = GetLastError()) TRACE("OpenPort Entry Error = %x\n", Error); //pmt


	// Make sure data sync mode is set correctly
	SetDataSyncMode(DATA_MODE_STOP);

	
	// Wait for single object waits in case a close port delete
	// operation is still pending.
	//pmt ::WaitForSingleObject(m_hDeletePortDoneEvent, INFINITE);
	if (long Error = GetLastError()) TRACE("OpenPort 1 Error = %x\n", Error); //pmt
	
	switch(::WaitForSingleObject(m_hDeletePortDoneEvent, NEAR_INFINITE))
	{
		case WAIT_OBJECT_0:		// The object was signaled within the time-out. 
			break;					// The normal case, just continue.
		
		case WAIT_ABANDONED:	// mutex object was not released by the ended thread 
		case WAIT_TIMEOUT:		// The time-out interval elapsed 
		default:
			liErrorCode = ERROR_WAIT_DELETE_PORT_DONE;
			break;
	}

	if ((liErrorCode == 0) && (m_pPort == 0))
	{
		// Flag for opening
		m_iIsOpening = 1;


		// COM port string passed to function
		sprintf(m_szCOM, "%s", lpszCOMPort);
		len = strlen(m_szCOM);

		// How many digits after COM
		if(len == 4)
		{
			// COMX (X = 1-9)
			sprintf(m_szCOM_long, "%s", m_szCOM);
		}
		else if(len > 4)
		{
			// COMXXX (XXX = 10-256)
			sprintf(m_szCOM_long, "\\\\.\\%s", m_szCOM);
		}
		else
		{
			// Error condition
			liErrorCode = ERROR_OPEN_PORT_COM_STRING;
		}


		// If no error
		if(liErrorCode == 0)
		{
			// Create CDialog class

			// Create port object
			m_pPort = new MyWin32Port(
						NULL,							// Send RS232 messages this window
						(const char *) m_szCOM_long,	// Com port to open (string)
						liBaudRate,						// Baud rate (integer)
						lpszParity[0],					// Parity (one letter)
						liWordSize,						// Usually 8 bits (integer)
						liStopBits,						// Usually 1 bit (integer)
						UNCHANGED,						// Undefined
						UNCHANGED,						// Undefined
						liXonXoff,						// Flow control
						liRtsCts,						// Rarely used
						liDtrDsr						// Rarely used
						);

			if (long Error = GetLastError()) 
			{
				if (Error != ERROR_FILE_NOT_FOUND)		// Value 2. Not yet an error??
				{
					TRACE("OpenPort 2 Error = %x\n", Error); //pmt
				}
				else SetLastError(0);					// Clear it for clarity later
			}

			// Set default timeout value
			m_pPort->SetRS232Timeout(m_iTimeout);

			if (long Error = GetLastError()) TRACE("OpenPort 3 Error = %x\n", Error); //pmt

			// Load error code if any
			liErrorCode = m_pPort->ErrorStatus();

			if (long Error = GetLastError()) TRACE("OpenPort 4 Error = %x\n", Error); //pmt

			// Check for success
			if(liErrorCode == ERROR_RS232_SUCCESS)
			{
				// Retrieve handle for packet ready event
				m_hPacketReadyEvent = m_pPort->GetPacketReadyEvent();


				// pmt ::Sleep(OPEN_PORT_WAIT);
				// pmt was Reset device
				// pmt liErrorCode = m_pPort->RequestReset(m_iTimeout);

				// Send packet request command
				liErrorCode = m_pPort->RequestPacket(m_pPort->GetTimeout());
				if (long Error = GetLastError()) TRACE("OpenPort 5 Error = %x\n", Error); //pmt

			}
			else
			{
				iDestroyObject = 1;

				// Error code already set by ErrorStatus()
			}
		}


		if(iDestroyObject == 1)
		{
			// Delete and clear object if there was an error
			if(m_pPort != 0)
			{
				// Delete port objects
				DeletePort();
				if (long Error = GetLastError()) TRACE("OpenPort 6 Error = %x\n", Error); //pmt
			}
		}


		// Flag off for opening
		m_iIsOpening = 0;
	}
	else
	{
		if (long Error = GetLastError()) TRACE("OpenPort 7 Error = %x\n", Error); //pmt
		// A COM port is already open
		liErrorCode = ERROR_OPEN_PORT_OPEN_PORT;
	}


	return(liErrorCode);
} // end CSQInclApp::OpenPort


//**************************************************
LONG CSQInclApp::CloseCOMPort(LPSTR lpszCOMPort)
{
	LONG liErrorCode = 0;

	if (long Error = GetLastError()) TRACE("CloseCOMPort A Error = %x\n", Error); //pmt

	// If port is open
	if(m_pPort != 0)
	{
		// If trying to close correct COM port
		if(strcmp(m_szCOM, lpszCOMPort) == 0)
		{
			//pmt::Sleep(CLOSE_PORT_WAIT);

			// Delete port objects
			DeletePort();
			if (long Error = GetLastError()) TRACE("CloseCOMPort B Error = %x\n", Error); //pmt

			//pmt::Sleep(CLOSE_PORT_WAIT);
		}
		else
		{
			liErrorCode = ERROR_CLOSE_PORT_COM_STRING;
		}
	}
	else
	{
		liErrorCode = ERROR_CLOSE_PORT_PORT_NOT_OPEN;
	}


	return(liErrorCode);
} // End CloseCOMPort()

//*************************************
void CSQInclApp::CloseCurrentPort(void)
{
	// If port is open
	if(m_pPort != 0)
	{
		// Delete port objects
		DeletePort();
	}
}


int CSQInclApp::GetDeletePortRequest(void)
{
	return(m_iDeletePortRequest);
}


void CSQInclApp::SetDeletePortRequestOn(void)
{
	m_iDeletePortRequest = 1;
}


void CSQInclApp::SetDeletePortRequestOff(void)
{
	m_iDeletePortRequest = 0;
}

//*****************************************************************8
void CSQInclApp::DeletePort(void)
{
	if (long Error = GetLastError()) TRACE("::DeletePort Error on Entry = %x\n", Error); //pmt

	ASSERT(m_pPort != 0);

	// Flag for closing
	m_iIsClosing = 1;

	
	::ResetEvent(m_hDeletePortDoneEvent);
	SetDeletePortRequestOn();
	int iSyncMode = GetDataSyncMode();
	SetDataSyncMode(DATA_MODE_STOP);
	if (long Error = GetLastError()) TRACE("::DeletePort Error B = %x\n", Error); //pmt
	
	// pmt
	m_pPort->FlushTXBuffer();
	m_pPort->FlushRXBuffer();
	m_pPort->SQFlushRX();

	if (iSyncMode == DATA_MODE_SYNC)
	{
		::SetEvent(m_hPacketReadyEvent);
		if (long Error = GetLastError()) TRACE("::SetEvent(m_hPacketReadyEvent) Returned = %x\n", Error); //pmt
	}
	else if((iSyncMode == DATA_MODE_ASYNC_NO_EVENT) ||
		    (iSyncMode == DATA_MODE_ASYNC_EVENT) )
	{
		if (long Error = GetLastError()) TRACE("::DeletePort Error C = %x\n", Error); //pmt
		// Wait for state machine to signal
		// pmt Is this because the calling program may somewhere
		//    be waiting for a packet to complete??
		//    The HandleRxNotify() routine, run at a completion of a read,
		//    Sets the DeletePortEvent - but if nothing is received, the program
		//    would hang on the next line.
		//    ::WaitForSingleObject(m_pPort->GetDeletePortEvent(), INFINITE);
		switch(long Result = ::WaitForSingleObject(m_pPort->GetDeletePortEvent(), m_pPort->GetTimeout()))
		{
			case WAIT_OBJECT_0:		// The object was signaled within the time-out. 
				break;				// The normal case, just continue.
			
			case WAIT_ABANDONED:	// mutex object was not released by the ended thread 
			case WAIT_TIMEOUT:		// The time-out interval elapsed 
			default:
				//liErrorCode = ERROR_WAIT_DELETE_PORT_DONE;
				TRACE("Wait for m_pPort->GetDeletePortEvent failed: %x\n", Result);
				break;
		}
	}
	else
	{
		// NB: pmt did this above before cancelling threads
		m_pPort->SQFlushRX();
	}


	// Pre-delete dependent objects
	m_pPort->PreDeleteObjects();

	// Delete class object
	delete(m_pPort);


	m_pPort = 0;


	SetDeletePortRequestOff();


	// Tell other functions we're ready (open port for example)
	::SetEvent(m_hDeletePortDoneEvent);


	// Flag off for closing
	m_iIsClosing = 0;

} // End DeletePort()


//********************************************
LONG CSQInclApp::ResetDevice(LPSTR lpszCOMPort)
{
	LONG liErrorCode = 0;


	// If port is open
	if(m_pPort != 0)
	{
		// If trying to access correct COM port
		if(strcmp(m_szCOM, lpszCOMPort) == 0)
		{
			FLOAT fPitch;
			FLOAT fRoll;
			LONG liPacketCount;
			LONG liPacketError;
			LONG liLatchedError;
			LONG liBytesAvail;
			int liMode;


			// Get data acquisition sync mode
			liMode = GetDataSyncMode();


			// If currently in async mode
			if((liMode == DATA_MODE_ASYNC_NO_EVENT) ||
				(liMode == DATA_MODE_ASYNC_EVENT))
			{
				SetDataSyncMode(DATA_MODE_STOP);

				m_pPort->FlushTXBuffer();
				m_pPort->FlushRXBuffer();
				m_pPort->SQFlushRX();

				SetDataSyncMode(DATA_MODE_SYNC);
			}


			// Carry out two synchronous get data's
			liErrorCode = GetData2(
				lpszCOMPort,
				&fPitch,
				&fRoll,
				&liPacketCount,
				&liPacketError,
				&liLatchedError,
				&liBytesAvail);


			// The first call to get data defines the error
			// code for this function.  Subsequent calls
			// will always time out.
			if(liErrorCode == ERROR_GET_DATA_WAIT_OBJECT)
			{
				// If the first call to get data timed out
				// then the device is probably powered off
				// or disconnected.
				liErrorCode = ERROR_RESET_DEVICE_TIMEOUT;
			}


			// Flag for reset
			m_pPort->SetResetFlagOn();


			GetData2(
				lpszCOMPort,
				&fPitch,
				&fRoll,
				&liPacketCount,
				&liPacketError,
				&liLatchedError,
				&liBytesAvail);



			// Set reset flag off
			m_pPort->SetResetFlagOff();


			// Clear buffers
			SetDataSyncMode(DATA_MODE_STOP);

			m_pPort->FlushTXBuffer();
			m_pPort->FlushRXBuffer();
			m_pPort->SQFlushRX();

			SetDataSyncMode(DATA_MODE_SYNC);


			// Wait for a successful packet to be read
			long liTime1, liTime2, liDiffTime;
			liTime1 = ReadTime();
			liDiffTime = 0;
			int iWait = -1;
			int liLastPacket;
			int liDiffPacket;

			iWait = GetData2(
				lpszCOMPort,
				&fPitch,
				&fRoll,
				&liPacketCount,
				&liPacketError,
				&liLatchedError,
				&liBytesAvail);

			liLastPacket = liPacketCount;
			liDiffPacket = 0;

			while(iWait != 0)
			{
				iWait = GetData2(
					lpszCOMPort,
					&fPitch,
					&fRoll,
					&liPacketCount,
					&liPacketError,
					&liLatchedError,
					&liBytesAvail);

				// Make sure packet count is incrementing
				liDiffPacket = liPacketCount - liLastPacket;

				if(liDiffPacket != 0)
				{
					iWait = 0;
					liErrorCode = 0;
				}

				// Degenerate timeout case (device wire cut)
				if(liDiffTime > RESET_DEVICE_TIMEOUT)
				{
					iWait = 0;
					liErrorCode = ERROR_RESET_DEVICE_TIMEOUT;
				}
				else
				{
					liTime2 = ReadTime();
					liDiffTime = liTime2 - liTime1;
				}
			}


			// Reset packet counter
			m_pPort->ResetPacketCount();


			// If previously in async mode
			if((liMode == DATA_MODE_ASYNC_NO_EVENT) ||
				(liMode == DATA_MODE_ASYNC_EVENT))
			{
				// Return to original mode
				SetDataSyncMode(liMode);

				// Send packet request command
				m_pPort->RequestPacket(m_iTimeout);
			}
		}
		else
		{
			liErrorCode = ERROR_RESET_DEVICE_COM_STRING;
		}
	}
	else
	{
		liErrorCode = ERROR_RESET_DEVICE_PORT_NOT_OPEN;
	}


	return(liErrorCode);
}


LONG CSQInclApp::GetErrorString(
	LONG liErrorCode,
	LONG liBuffSize,
	LPTSTR lptszBuffer)
{
	LONG liRetval;
	LONG liLocalErrorCode;


	liLocalErrorCode = 0;


	liRetval = ::LoadString(
		m_hInstance,
		-liErrorCode,
		lptszBuffer,
		ERROR_STRING_LENGTH);


	if(liRetval == 0)
	{
		liLocalErrorCode = -1;
	}

	
	return(liLocalErrorCode);
}


LONG CSQInclApp::ClearLatchedError(LPSTR lpszCOMPort)
{
	LONG liErrorCode = 0;


	// If port is open
	if(m_pPort != 0)
	{
		// If trying to access correct COM port
		if(strcmp(m_szCOM, lpszCOMPort) == 0)
		{
			// Call clear latched error
			m_pPort->ClearLatchedError();
		}
	}
	else
	{
		liErrorCode = ERROR_CLEAR_LATCHED_PORT_NOT_OPEN;
	}


	return(liErrorCode);
}


//*******************************************************************
LONG CSQInclApp::GetData2(
	LPSTR lpszCOMPort,
	FLOAT * pfValue1,
	FLOAT * pfValue2,
	LONG * pliPacketCount,
	LONG * pliPacketError,
	LONG * pliLatchedError,
	LONG * pliBytesInBuffer)
{
	// Required for MFC DLL's
	AFX_MANAGE_STATE(AfxGetStaticModuleState());


	LONG liErrorCode;					// Local error code
	unsigned short usPitch = 0.0;		// Pitch
	unsigned short usRoll = 0.0;		// Roll


	// Assume no error
	liErrorCode = 0;


	// If port is open
	if(m_pPort != 0)
	{
		// If trying to access correct COM port
		if(strcmp(m_szCOM, lpszCOMPort) == 0)
		{
			// If not trying to close port
			if(m_iIsClosing != 1)
			{
				// If not trying to open port
				if(m_iIsOpening != 1)
				{
					// If asynchronous mode is on
					int iCurrentSyncMode = GetDataSyncMode();

					// In case user forgot to set data sync mode
					// Default to synchronous
					if(iCurrentSyncMode == DATA_MODE_STOP)
					{
						SetDataSyncMode(DATA_MODE_SYNC);
					}

					if(iCurrentSyncMode == DATA_MODE_SYNC)
					{
						// Send packet request command
						liErrorCode = m_pPort->RequestPacket(
							m_pPort->GetTimeout());


						if(liErrorCode == 0)
						{
							// Wait here for packet read event
							DWORD dwResult;					// Result of wait
							DWORD dwMilliseconds;			// Time-out interval

							// Wait timeout set to default defined macro value
							dwMilliseconds = SYNC_GET_DATA_WAIT;

							// Wait for done event
							dwResult = ::WaitForSingleObject(
								m_hPacketReadyEvent,
								dwMilliseconds);

							// If wait was not successfull
							if(dwResult != WAIT_OBJECT_0)
							{
								liErrorCode = ERROR_GET_DATA_WAIT_OBJECT;
							}
						}
					} // End of SYNC mode


					if((iCurrentSyncMode == DATA_MODE_ASYNC_NO_EVENT) ||
						(iCurrentSyncMode == DATA_MODE_ASYNC_EVENT))
					{
						// If first call to async get data
						if(m_iAsyncFirstTime == 1)
						{
							// Force reset to trigger async state machine
							m_pPort->RequestPacket(m_iTimeout);

							m_iAsyncFirstTime = 0;
						}
					}


					if(liErrorCode == 0)
					{
						// Packet is ready to be read
						// Read thread is paused for DLL to read
						m_pPort->CopyCurrentPacket(&m_CurrentPacket);


						// Send data back to user
						*pliPacketCount = m_CurrentPacket.usPacketCount;
						*pliPacketError = (LONG)m_CurrentPacket.ssCurrentErrorState;
						*pliLatchedError = m_CurrentPacket.usLatchedErrorState;

						// If no errors in read thread
						liErrorCode = (LONG)m_CurrentPacket.ssCurrentErrorState;

						if(liErrorCode == 0)
						{
							// Return pitch as value 1
							*pfValue1 =
								((FLOAT)(m_CurrentPacket.usValue1)) / 10.0;

							// Return roll as value 2
							*pfValue2 =
								((FLOAT)(m_CurrentPacket.usValue2)) / 10.0;
						}
					}
				}
				else
				{
					liErrorCode = ERROR_GET_DATA_PORT_OPENING;
				}
			}
			else
			{
				liErrorCode = ERROR_GET_DATA_PORT_CLOSING;
			}
		}
		else
		{
			liErrorCode = ERROR_GET_DATA_COM_STRING;
		}
	}
	else
	{
		liErrorCode = ERROR_GET_DATA_PORT_NOT_OPEN;
	}


	return(liErrorCode);
} // End GetData2()


int CSQInclApp::GetDataSyncMode(void)
{
	return(m_iDataSyncMode);
}


void CSQInclApp::SetDataSyncMode(int iSyncMode)
{
	// Update data sync mode flag
	m_iDataSyncMode = iSyncMode;
}


void CSQInclApp::SetPollingModeOn(void)
{
	m_iPollingMode = 1;
}


void CSQInclApp::SetPollingModeOff(void)
{
	m_iPollingMode = 0;
}


int CSQInclApp::GetPollingMode(void)
{
	return(m_iPollingMode);
}


//***********************************************************
LONG CSQInclApp::FindFirstDevice(
	LONG liStartCOM,
	LONG liEndCOM,
	LONG liBaudRate,
	LPSTR lpszParity,
	LONG liWordSize,
	LONG liStopBits,
	LONG liXonXoff,
	LONG liRtsCts,
	LONG liDtrDsr)
{
	// Required for MFC DLL's
	AFX_MANAGE_STATE(AfxGetStaticModuleState());


	LONG liErrorCode = 0;
	if (long Error = GetLastError()) TRACE("FindDevice 0 Error = %x\n", Error); //pmt


	if(m_pPort == 0)
	{
		LONG liRetval = 0;
		LONG liNextCOM = 0;
		DWORD dwResult;					// Result of wait
		DWORD dwMilliseconds;			// Time-out interval


		//SetPollingModeOn();


		// Loop counters and flags
		int iLooping = 1;
		int iCurrentCOM = liStartCOM;


		// Loop through range of COM ports
		while(iLooping == 1)
		{
			// COM port string passed to function
			sprintf(m_szCOM, "COM%d", iCurrentCOM);

			// Open current COM port
			liErrorCode = OpenPort(
									m_szCOM,
									liBaudRate,
									lpszParity,
									liWordSize,
									liStopBits,
									liXonXoff,
									liRtsCts,
									liDtrDsr);
			if (long Error = GetLastError()) TRACE("FindDevice 1 Error = %x\n", Error); //pmt


			// Check for success
			if(liErrorCode == ERROR_RS232_SUCCESS)
			{
				SetPollingModeOn();					// Tell RxNotify to request packet
				// Wait here for packet read event

				// Wait timeout set to default defined macro value
				dwMilliseconds = FIND_DEVICE_WAIT;

				// Wait for done event
				dwResult = ::WaitForSingleObject(m_hPacketReadyEvent, dwMilliseconds);

				// If wait was successfull
				if(dwResult == WAIT_OBJECT_0)
				{
					iLooping = 0;
				}

				SetPollingModeOff();				// Send no more packets

				// Close port
				if (long Error = GetLastError()) TRACE("FindDevice 2 Error = %x\n", Error); //pmt
				
				CloseCurrentPort();

			} // End if OpenPort succeeded

			if(iLooping == 1)
			{
				iCurrentCOM++;

				if(iCurrentCOM > liEndCOM)
				{
					iLooping = 0;
				}
			}
		}

		//pmt SetPollingModeOFF();



		//pmt::Sleep(CLOSE_PORT_WAIT);


		if(iCurrentCOM <= liEndCOM)
		{
			// Device was found, return COM index
			liErrorCode = iCurrentCOM;
		}
		else
		{
			// Device was not found in this range of COM indices
			liErrorCode = ERROR_FIND_DEVICE_NOT_FOUND;
		}
	}
	else
	{
		// A COM port is already open
		liErrorCode = ERROR_FIND_DEVICE_OPEN_PORT;
	}

	SetLastError(0);		// So we don't have a "File not found" error left set
	if (long Error = GetLastError()) TRACE("FindDevice Exit Error = %x\n", Error); //pmt

	return(liErrorCode);
} // End FindFirstDevice()


//*******************************
LONG CSQInclApp::GetCTSStatus(
	LPSTR lpszCOMPort,
	LONG * pliCTSStatus)
{
	LONG liErrorCode = 0;


	// If port is open
	if(m_pPort != 0)
	{
		// If trying to access correct COM port
		if(strcmp(m_szCOM, lpszCOMPort) == 0)
		{
			*pliCTSStatus = m_pPort->GetCTSStatus();
		}
		else
		{
			liErrorCode = ERROR_GET_CTS_STATUS_COM_STRING;
		}
	}
	else
	{
		// A COM port is not open
		liErrorCode = ERROR_GET_CTS_STATUS_PORT_NOT_OPEN;
	}


	return(liErrorCode);
}


LONG CSQInclApp::GetDSRStatus(
	LPSTR lpszCOMPort,
	LONG * pliDSRStatus)
{
	LONG liErrorCode = 0;


	// If port is open
	if(m_pPort != 0)
	{
		// If trying to access correct COM port
		if(strcmp(m_szCOM, lpszCOMPort) == 0)
		{
			*pliDSRStatus = m_pPort->GetDSRStatus();
		}
		else
		{
			liErrorCode = ERROR_GET_DSR_STATUS_COM_STRING;
		}
	}
	else
	{
		// A COM port is not open
		liErrorCode = ERROR_GET_DSR_STATUS_PORT_NOT_OPEN;
	}


	return(liErrorCode);
}


//****************************************************************
LONG CSQInclApp::SetRTSStatus(LPSTR lpszCOMPort, LONG liRTSStatus)
{
	LONG liErrorCode = 0;


	// If port is open
	if(m_pPort != 0)
	{
		// If trying to access correct COM port
		if(strcmp(m_szCOM, lpszCOMPort) == 0)
		{
			m_pPort->SetRTSStatus(liRTSStatus);
		}
		else
		{
			liErrorCode = ERROR_SET_RTS_STATUS_COM_STRING;
		}
	}
	else
	{
		// A COM port is not open
		liErrorCode = ERROR_SET_RTS_STATUS_PORT_NOT_OPEN;
	}


	return(liErrorCode);
}


LONG CSQInclApp::SetDTRStatus(
	LPSTR lpszCOMPort,
	LONG liDTRStatus)
{
	LONG liErrorCode = 0;


	// If port is open
	if(m_pPort != 0)
	{
		// If trying to access correct COM port
		if(strcmp(m_szCOM, lpszCOMPort) == 0)
		{
			m_pPort->SetDTRStatus(liDTRStatus);
		}
		else
		{
			liErrorCode = ERROR_SET_DTR_STATUS_COM_STRING;
		}
	}
	else
	{
		// A COM port is not open
		liErrorCode = ERROR_SET_DTR_STATUS_PORT_NOT_OPEN;
	}


	return(liErrorCode);
}


/////////////////////////////////////////////////////////////////////////////
// The one and only CSQInclApp object

CSQInclApp theApp;


/////////////////////////////////////////////////////////////////////////////
// Event handlers

BOOL CSQInclApp::InitInstance() 
{
	// Object Creation and Initialization
	CreateDataObjects();

	
	return CWinApp::InitInstance();
}

int CSQInclApp::ExitInstance() 
{
	// Object destruction
	DestroyDataObjects();

	
	return CWinApp::ExitInstance();
}
