/****************************************************************
*
* Copyright (C) 2004 SignalQuest, Inc.
*
*----------------------------------------------------------------
* All software and related documentation is provided "AS IS" and 
* without warranty or support of any kind and SignalQuest 
* expressly disclaims all other warranties, express or implied, 
* including, but not limited to, the implied warranties of 
* merchantability and fitness for a particular purpose. Under no 
* circumstances shall SignalQuest be liable for any 
* incidental, special or consequential damages that result from 
* the use or inability to use the software or related 
* documentation, even if SignalQuest has been advised of 
* the liability.
*
* Unless otherwise stated, software written and copyrighted by 
* SignalQuest is distributed as "freeware". You may use 
* and modify this software without any charge or restriction. 
* You may distribute to others, as long as the original author 
* is acknowledged.
*
*****************************************************************
* 
* Project:  SignalQuest Serial Communications DLL Demo Program in VC++
*
* File:    TestSQinclDlg.h
*
* History:
*   Version 1.71 (2004-11-11) pmt. Extensive fixes to thread usage.
*   Version 1.5  (2004-08-30)
*
****************************************************************/


// TestSQInclDlg.h : header file
//

#if !defined(AFX_TESTSQINCLDLG_H__41CF4510_895E_4600_A1CF_A8B21F0394B9__INCLUDED_)
#define AFX_TESTSQINCLDLG_H__41CF4510_895E_4600_A1CF_A8B21F0394B9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000



#define TIMER_ID					1
#define LOOP_TIMER_DEFAULT			15
#define RS232TIMEOUT				75
#define RXTXDELAY					30
#define ASYNC_EVENT_WAIT			2000
#define DATA_IO_TIMEOUT				3000
#define STATUS_TEXT_LINES			40

#define MODE_LOAD_DLL				-1
#define MODE_STOP					0
#define MODE_SYNC					1
#define MODE_ASYNC_NO_EVENT			2
#define MODE_ASYNC_EVENT			3




/////////////////////////////////////////////////////////////////////////////
// CTestSQInclDlg dialog

typedef struct tagWritePacket
{
	FLOAT fPitch;
	FLOAT fRoll;
	LONG liPacketCount;
	LONG liErrorCode;
} WRITEPACKET, *PWRITEPACKET;


class CTestSQInclDlg : public CDialog
{
// Construction
public:
	CTestSQInclDlg(CWnd* pParent = NULL);	// standard constructor
	//~CTestSQInclDlg();

// Dialog Data
	//{{AFX_DATA(CTestSQInclDlg)
	enum { IDD = IDD_TESTSQINCL_DIALOG };
	CButton	m_cbToggleRtsStatus;
	CButton	m_cbToggleDtrStatus;
	CEdit	m_ceRtsStatus;
	CEdit	m_ceDtrStatus;
	CEdit	m_ceDsrStatus;
	CEdit	m_ceCtsStatus;
	CButton	m_cbToggleCtsStatus;
	CButton	m_cbOpenClosePort;
	CEdit	m_Points2Acquire;
	CListBox	m_lbStatus;
	CComboBox	m_cbSyncMode;
	CEdit	m_DuplicatePackets;
	CEdit	m_LatchedError;
	CEdit	m_DroppedPackets;
	CEdit	m_PacketCount;
	CEdit	m_LoopTimer;
	CEdit	m_COMPort;
	BOOL	m_SaveToDisk;
	UINT	m_Repetitions;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTestSQInclDlg)
	public:
	virtual BOOL DestroyWindow();
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL


	// User-defined variables
protected:
	HICON m_hIcon;
	UINT m_nTimer;
	int m_iPortIsOpen;
	int m_iDoGetData;

	unsigned short m_usPrevPacketCount;
	unsigned short m_usDroppedPackets;
	unsigned short m_usLatchedError;
	unsigned short m_usDuplicatePackets;
	unsigned short m_usThreeOrMore;

	CFile * m_pFile;
	char m_szFileName[256];

	HANDLE m_hPacketReadEvent;

	int m_iDataSyncMode;
	int m_iStatusLineCount;
	int m_iIsStreaming;

	int m_iDsrStatus;
	int m_iCtsStatus;
	int m_iDtrStatus;
	int m_iRtsStatus;


	// User-defined functions
protected:
	void InitVars(void);
	int OpenPort(int iOptCom);
	int GetData(int iOptCom);
	int ClosePort(int iOptCom);
	int GetCOMPort(LONG * pliCOMPort);
	void Shutdown(void);
	void OpenDiskFile(void);
	void CloseDiskFile(void);
	void WriteToDiskFile(PWRITEPACKET pPacket);
	int FindFirstDevice(int iComLo, int iComHi);
	void SetStatusText(LPCSTR lpszString);
	void StopLoopData(void);
	void DataModeSwitch(void);
	void DoStartDataLoop(void);
	void DoStopDataLoop(void);
	void DisplayErrorText(LONG liErrorCode);
	void DoLinearTest(void);
	void DoOpenCloseTest(void);
	void DoReadDsr(void);
	void DoReadCts(void);
	LONG DoWriteDtr(LONG liDtrStatus);
	LONG DoWriteRts(LONG liRtsStatus);



	// Generated message map functions
	//{{AFX_MSG(CTestSQInclDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	virtual void OnCancel();
	virtual void OnOK();
	afx_msg void OnOpenPort();
	afx_msg void OnGetData();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnClearLatchedError();
	afx_msg void OnSendReset();
	afx_msg void OnSetFile();
	afx_msg void OnSaveToDisk();
	afx_msg void OnLinearTest();
	afx_msg void OnStartDataLoop();
	afx_msg void OnStopDataLoop();
	afx_msg void OnFindFirstDevice();
	afx_msg void OnToggleDtrStatus();
	afx_msg void OnToggleRtsStatus();
	afx_msg void OnGetCtsStatus();
	afx_msg void OnGetDsrStatus();
	afx_msg void OnKillfocusRepetitions();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void DispatchMessages(void);
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TestSQInclDLG_H__41CF4510_895E_4600_A1CF_A8B21F0394B9__INCLUDED_)
